package com.ruoyi.project.hit.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.CustomException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.project.hit.controller.vo.ListCardGameVo;
import com.ruoyi.project.hit.domain.CardGame;
import com.ruoyi.project.hit.domain.CardGameRule;
import com.ruoyi.project.hit.service.CardGameProductService;
import com.ruoyi.project.hit.service.CardGameRuleService;
import com.ruoyi.project.hit.service.CardGameService;
import com.ruoyi.project.hit.mapper.CardGameMapper;
import com.ruoyi.project.hit.service.CardUserHitService;
import com.ruoyi.project.uniapp.controller.vo.CardProductVo;
import com.ruoyi.project.uniapp.controller.vo.CardUserGameVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
* @author HP
* @description 针对表【card_game】的数据库操作Service实现
* @createDate 2024-05-25 21:09:47
*/
@Slf4j
@Service
public class CardGameServiceImpl extends ServiceImpl<CardGameMapper, CardGame>
    implements CardGameService{
    @Autowired
    private RedisTemplate redisTemplate;

    @Resource
    private CardGameMapper cardGameMapper;
    @Resource
    private CardGameRuleService cardGameRuleService;
    @Resource
    private CardGameProductService cardGameProductService;
    @Resource
    private CardUserHitService cardUserHitService;

    @Override
    public List<HashMap> selectCardGameList(ListCardGameVo listCardGameVo) {
        return cardGameMapper.selectCardGameList(listCardGameVo);
    }

    @Override
    public int addCardGame(CardGame cardGame) {
        LambdaQueryWrapper<CardGame> wrapper=new LambdaQueryWrapper<>();
        wrapper.eq(CardGame::getTitle,cardGame.getTitle());
        CardGame cardGame1 = cardGameMapper.selectOne(wrapper);
        if(null!=cardGame1){
            throw new CustomException("该活动标题已存在,添加失败");
        }
        //活动状态  0:未开始  1:已结束(已加载)
        if(cardGame.getStartTime().after(new Date())){
            cardGame.setStatus(0);
        }else {
            cardGame.setStatus(1);
        }
        return cardGameMapper.insert(cardGame);
    }

    @Override
    public Map<String, Object> getCardGameDataById(Integer id) {
        Map<String,Object> map=new HashMap<>();

        List<HashMap> list1 = cardGameRuleService.getGameRuleListByGameId(id);
        map.put("gameRuleList",list1);
        List<HashMap> list2=cardGameProductService.getGameProductByGameId(id);
        map.put("gameProductList",list2);
        CardGame cardGame = cardGameMapper.selectById(id);
        map.put("cardGame",cardGame);
        return map;
    }

    @Override
    public int updateCardGame(CardGame cardGame) {
        LambdaQueryWrapper<CardGame> wrapper =new LambdaQueryWrapper<>();
        wrapper.eq(CardGame::getTitle,cardGame.getTitle());
        wrapper.ne(CardGame::getId,cardGame.getId());
        CardGame cardGame1 = cardGameMapper.selectOne(wrapper);
        if(cardGame1!=null){
            throw new CustomException("该活动标题已存在，修改失败");
        }
        return cardGameMapper.updateById(cardGame);
    }

    @Override
    public int deleteCardGameById(Integer id) {
        CardGame cardGame = cardGameMapper.selectById(id);
        //如果活动已开始，则不可删除
        if(cardGame.getStartTime().before(new Date())){
            throw new CustomException("活动已开始，不可执行删除操作");
        }
        //删除对应活动奖品配置信息
        cardGameProductService.deleteByGameId(id);
        //删除对应活动配置的活动奖品策略
        cardGameRuleService.deleteByGameId(id);
        return cardGameMapper.deleteById(id);
    }

    @Override
    public Map<String, Object> getCardGameDetailById(Integer id) {
        Map<String,Object> map=new HashMap<>();

        List<HashMap> list1 = cardGameRuleService.getGameRuleListByGameId(id);
        map.put("gameRuleList",list1);
        List<HashMap> list2=cardGameProductService.getGameProductByGameId(id);
        map.put("gameProductList",list2);
        HashMap cardGame = cardGameMapper.getCardGameByGameId(id);
        map.put("cardGame",cardGame);
        return map;
    }

    @Override
    public CardGame getCardGameByGameId(Integer gameid) {
        return cardGameMapper.selectById(gameid);
    }

    @Override
    public List<CardGame> carousel() {
        LambdaQueryWrapper<CardGame> wrapper=new LambdaQueryWrapper<>();
        wrapper.select(CardGame::getPic);
        wrapper.select(CardGame::getTitle);
        return cardGameMapper.selectList(wrapper);
    }

    @Override
    public List<HashMap> showAllCardGame() {
        return cardGameMapper.showAllCardGame();
    }

    @Override
    public Map<String, Object> getGameDetail(Integer gameId) {
        Map map=new HashMap();
        List<HashMap> gameRuleList = cardGameRuleService.getGameRuleListByGameId(gameId);
        map.put("gameRuleList",gameRuleList);

        List<HashMap> gameProductList = cardGameProductService.getGameProductByGameId(gameId);
        map.put("gameProductList",gameProductList);
        Map<String, Object> map1 = cardUserHitService.selectCardUserHitInfoByGameId(gameId);
        map.putAll(map1);
        return map;
    }

    @Override
    public void preHot(Date now) {
        List<CardGame> cardGameList = cardGameMapper.getCardByDate(now);
        if(cardGameList.size()==0){
            return;
        }
        //活动信息缓存redis（json字符串存储）
        for (CardGame cardGame : cardGameList) {
            String jsonCardGame = JSONArray.toJSONString(cardGame);
            //失效时间
            long l=cardGame.getEndTime().getTime()-now.getTime();
            redisTemplate.opsForValue().set("cardGame:"+cardGame.getId(),jsonCardGame,l,TimeUnit.MILLISECONDS);

            List<CardGameRule> gameRuleList = cardGameRuleService.getGameRuleList(cardGame.getId());
            for (CardGameRule cardGameRule : gameRuleList) {
                //最大中奖数缓存redis（hash存储）
                redisTemplate.opsForHash().put("cardGame_maxHitCount:"+cardGame.getId(),cardGameRule.getLevelid(),cardGameRule.getHitCount());
                //最大抽奖数缓存redis（hash存储）
                redisTemplate.opsForHash().put("cardGame_maxEnterCount:"+cardGame.getId(),cardGameRule.getLevelid(),cardGameRule.getEnterCount());
            }
            //设置两个hash失效时间
            redisTemplate.expire("cardGame_maxHitCount:"+cardGame.getId(),l, TimeUnit.MILLISECONDS);
            redisTemplate.expire("cardGame_maxEnterCount:"+cardGame.getId(),l, TimeUnit.MILLISECONDS);

            List<CardProductVo> gameProductList = cardGameProductService.getGameProductList(cardGame.getId());
            //抽奖令牌桶
            List<Long> tokenList=new ArrayList<>();
            for (CardProductVo cardProductVo : gameProductList) {
                for (int i = 0; i < cardProductVo.getAmount(); i++) {
                    //给每个奖品都配置一个时间戳
                    Long start=cardGame.getStartTime().getTime();
                    Long end=cardGame.getEndTime().getTime();
                    //时间差
                    int time= (int) (end-start);
                    Long token=start+new Random().nextInt(time);
                    //避免重复
                    token=token*10000+new Random().nextInt(9999);
                    tokenList.add(token);
                    //奖品-令牌缓存redis(string存储）
                    redisTemplate.opsForValue().set("gameId_token:"+cardGame.getId()+"_"+token,cardProductVo,l,TimeUnit.MILLISECONDS);
                }

            }
            //由小到大排
            Collections.sort(tokenList);
            //令牌集合redis缓存（list存储）
            redisTemplate.opsForList().rightPushAll("game_token:"+cardGame.getId(),tokenList);
            redisTemplate.expire("game_token:"+cardGame.getId(),l,TimeUnit.MILLISECONDS);

            //活动状态置1
            cardGame.setStatus(1);
            cardGameMapper.updateById(cardGame);
        }


    }

    @Override
    public HashMap getCardGameBygameid(Integer gameid) {
        return cardGameMapper.getCardGameBygameid(gameid);
    }
}




