package com.ruoyi.project.hit.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.exception.CustomException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import com.ruoyi.project.hit.controller.vo.ListInfoVo;
import com.ruoyi.project.hit.controller.vo.TotalInfoVo;
import com.ruoyi.project.hit.domain.*;
import com.ruoyi.project.hit.service.CardGameService;
import com.ruoyi.project.hit.service.CardUserHitService;
import com.ruoyi.project.hit.mapper.CardUserHitMapper;
import com.ruoyi.project.hit.service.CardUserService;
import com.ruoyi.project.uniapp.controller.vo.CardProductVo;
import com.ruoyi.project.uniapp.controller.vo.CardUserGameVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
* @author HP
* @description 针对表【card_user_hit】的数据库操作Service实现
* @createDate 2024-05-28 23:47:42
*/
@Slf4j
@Service
public class CardUserHitServiceImpl extends ServiceImpl<CardUserHitMapper, CardUserHit>
    implements CardUserHitService{
    @Resource
    private CardUserHitMapper cardUserHitMapper;

    @Autowired
    private CardGameService cardGameService;
    @Autowired
    private CardUserService cardUserService;

    @Autowired
    private RedisTemplate redisTemplate;
    @Autowired
    private RabbitTemplate rabbitTemplate;

    @Override
    public List<HashMap> selectCardUserHitInfo(TotalInfoVo totalInfoVo) {
        return cardUserHitMapper.selectCardUserHitInfo(totalInfoVo);
    }

    @Override
    public Map<String,Object> selectCardUserHitInfoByGameId(Integer gameid) {
        Map<String,Object> map=new HashMap<>();
        //根据活动id拿到活动详情以及活动类型（2表）
        Map map1 = cardGameService.getCardGameBygameid(gameid);
//        log.debug("cardGame",map1);
        map.put("cardGame",map1);
        CardGame cardGame = cardGameService.getCardGameByGameId(gameid);
        if(StringUtils.isNotNull(cardGame)&&cardGame.getEndTime().before(new Date())){
            //根据活动id拿到活动中奖明细（3表）
            List<HashMap> hashMapList=cardUserHitMapper.getProductsBygameid(gameid);
            map.put("products",hashMapList);
        }
        return map;
    }

    @Override
    public List<HashMap> selectCardUserHitList(ListInfoVo listInfoVo) {
        return cardUserHitMapper.selectCardUserHitList(listInfoVo);
    }

    @Override
    public CardProductVo getPrizeDraw(CardUserGameVo cardUserGameVo) {
        Date now=new Date();
        boolean b1 = cardUserService.weatherLogin(cardUserGameVo.getUserId());
        if(!b1){
            throw new CustomException("用户未登录");
        }
        CardGame cardGame = JSON.parseObject((String) redisTemplate.opsForValue().get("cardGame:" + cardUserGameVo.getGameId()),CardGame.class);
        CardUser cardUser= (CardUser) redisTemplate.opsForValue().get("user_login:"+cardUserGameVo.getUserId());
        if(StringUtils.isNull(cardGame)||now.before(cardGame.getStartTime())){
            throw new CustomException("活动未开始");
        }
        if(now.after(cardGame.getEndTime())){
            throw new CustomException("活动已结束");
        }
        //判断该用户是否参与过抽奖
        Boolean b = redisTemplate.hasKey("cardGame_user:" + cardUserGameVo.getGameId() + "_" + cardUserGameVo.getUserId());
        if(!b){
            //value：抽奖次数
            redisTemplate.opsForValue().set("cardGame_user:" + cardUserGameVo.getGameId() + "_" + cardUserGameVo.getUserId(),0,(cardGame.getEndTime().getTime()-now.getTime()), TimeUnit.MILLISECONDS);
            //value：中奖次数
            redisTemplate.opsForValue().set("cardGame_user_hit:" + cardUserGameVo.getGameId() + "_" + cardUserGameVo.getUserId(),0,(cardGame.getEndTime().getTime()-now.getTime()), TimeUnit.MILLISECONDS);
        }
        int maxEnterCount = (int) redisTemplate.opsForHash().get("cardGame_maxEnterCount:" + cardUserGameVo.getGameId(), cardUser.getLevel());
        int EnterCount=(int) redisTemplate.opsForValue().get("cardGame_user:" + cardUserGameVo.getGameId() + "_" + cardUserGameVo.getUserId());
        if(EnterCount>=maxEnterCount){
            throw new CustomException("抽奖次数已达上限");
        }
        int maxHitCount = (int) redisTemplate.opsForHash().get("cardGame_maxHitCount:" + cardUserGameVo.getGameId(), cardUser.getLevel());
        int HitCount=(int) redisTemplate.opsForValue().get("cardGame_user_hit:" + cardUserGameVo.getGameId() + "_" + cardUserGameVo.getUserId());
        //抽奖记录次数加一
        redisTemplate.opsForValue().increment("cardGame_user:" + cardUserGameVo.getGameId() + "_" + cardUserGameVo.getUserId(),1);
        //mq写入抽奖记录到数据库
        CardGameUser cardGameUser=new CardGameUser(cardUserGameVo.getUserId(),cardUserGameVo.getGameId(),now);
        String cardGameUserStr= JSONObject.toJSONString(cardGameUser);
        rabbitTemplate.convertAndSend("drawRecord",cardGameUserStr);
        //该用户已经达到最大中奖次数
        if(HitCount>=maxHitCount){
            return null;
        }
        //抽奖流程
        Long token= (Long) redisTemplate.opsForList().leftPop("game_token:"+cardUserGameVo.getGameId());
        if(StringUtils.isNull(token)){
            throw new CustomException("奖品已抽完");
        }
        if(now.getTime()<token/10000){
            //没中奖,token放回令牌桶
            redisTemplate.opsForList().leftPush("game_token:"+cardUserGameVo.getGameId(),token);
            return null;
        }else {
            //中奖,中奖记录次数加一
            redisTemplate.opsForValue().increment("cardGame_user_hit:" + cardUserGameVo.getGameId() + "_" + cardUserGameVo.getUserId(),1);
            CardProductVo cardProductVo= (CardProductVo) redisTemplate.opsForValue().get("gameId_token:"+cardGame.getId()+"_"+token);
            //mq写入中奖信息到数据库
            CardUserHit cardUserHit=new CardUserHit(cardUserGameVo.getGameId(),cardUserGameVo.getUserId(),cardProductVo.getId(),now);
            String cardUserHitStr=JSONObject.toJSONString(cardUserHit);
            rabbitTemplate.convertAndSend("winnerRecord",cardUserHitStr);

            return cardProductVo;
        }

    }

    @Override
    public int addCardUserHit(CardUserHit cardUserHit) {
        return cardUserHitMapper.insert(cardUserHit);
    }

    @Override
    public List<HashMap> winnerRecord(Integer userId) {
        return cardUserHitMapper.winnerRecord(userId);
    }

}




